%define ALD_Ver		2.0

Name:         libffi

License:      MIT
Summary:      Library supporting Foreign Function Interfaces
Version:      3.1
Release:      0

#ftp://sourceware.org/pub/libffi/libffi-3.1.tar.gz

Source: %{name}-%{version}.tar.xz
Source2: %{name}-%{version}-patches.tgz

%define DestDir		/System
# -----------------------------------------------------

%if %{_target_cpu} == %{_build_cpu}
%define _srcrpmdir	%{_topdir}/SRPMS/%{ALD_Ver}
%define _CROSS 0
%define MakeLocCross 0
%else
# rpm --target=armv7hl -ba xxxxx.spec
%define _srcrpmdir	%{_topdir}/SRPMS/tmp
%define _CROSS 1
%define MakeLocCross 1
%define _host %{_arch}-%{_os}%{_gnu}
%endif

#------------------ CROSS DIR --------------------

%define docdir		%{_defaultdocdir}/%{name}

%define auto_local %{_sourcedir}/AUTO_LOCAL-1.sh
%define have_auto_local %(if test -x %{auto_local} ; then echo 1 ; else echo 0 ; fi)

%if %{have_auto_local}
%define __os_install_post %{nil}
%define CrossPath	%(%{auto_local} -GetCrossPath -build_cpu=%{_build_cpu} -target_cpu=%{_target_cpu})
%endif

# ------------------- description ------------------------------------
%description
Library supporting Foreign Function Interfaces
#-------------------------------
%package devel
Summary:      Libraries for the C compiler
%description devel
These libraries are needed to compile C code.
#-------------------------------
%package info
Summary:      Info for %{name}
%description info
Info for %{name}
# --------------------------------------
%package devel-man
Summary:      man3 for Libraries
%description devel-man
man3 for Libraries

# ------------------- setup ------------------------------------------
%Prep
%setup -q -n %{name}-%{version}

tar -xaf %SOURCE2
patch -p1<patches/libffi-3.1-fix-include-path.patch
patch -p1<patches/libffi-3.1-fix-exec-stack.patch
patch -p1<patches/libffi-aarch64-rhbz1174037.patch
patch -p1<patches/libffi-3.1-aarch64-fix-exec-stack.patch
#patch -p1<patches/libffi-3.1-libffi_tmpdir.patch

# ------------------- Build ------------------------------------------
%Build 
%if %{_CROSS}
export PATH="%{CrossPath}/bin:$PATH"
export GCC_NoHostPath=d
%endif

%if %{_arch} == "i386"
OPTFLAGS="-O2 -march=i586 -falign-functions=4 -falign-jumps=1 -falign-loops=1 -fomit-frame-pointer"
%endif
%if %{_arch} == "x86_64"
OPTFLAGS="-O2"
%endif
%if %{_arch} == "arm"
OPTFLAGS="%{optflags} -O2 -munaligned-access"
%endif

OPTFLAGS="$OPTFLAGS -ffunction-sections -fdata-sections -fno-asynchronous-unwind-tables"

CFLAGS="$OPTFLAGS -std=gnu99"
CXXFLAGS="$OPTFLAGS"
LDFLAGS="-Wl,--gc-sections"

%configure \
	--enable-shared \
	--disable-static \
	--disable-libtool-lock \
	--disable-debug \
	&>FConfigure_log.txt

make &>FMake_log.txt

# ------------------- Install ----------------------------------------
%Install
%if %{_CROSS}
export PATH="%{CrossPath}/bin:$PATH"
export GCC_NoHostPath=d
%endif

rm -rf $RPM_BUILD_ROOT
mkdir -p %{_srcrpmdir}
mkdir -p $RPM_BUILD_ROOT%{docdir}

make DESTDIR=$RPM_BUILD_ROOT install &>FInstall_log.txt
rm $RPM_BUILD_ROOT%{_libdir}/*.la
rm $RPM_BUILD_ROOT%{_infodir}/dir

cp LICENSE README $RPM_BUILD_ROOT%{docdir}
mv $RPM_BUILD_ROOT%{_libdir}/%{name}-%{version}/include $RPM_BUILD_ROOT%{_prefix}
rm -r $RPM_BUILD_ROOT%{_libdir}/%{name}-%{version}

%if %{_arch} == "x86_64"
  mv $RPM_BUILD_ROOT%{_includedir}/ffi.h $RPM_BUILD_ROOT%{_includedir}/ffi-64.h
  mv $RPM_BUILD_ROOT%{_includedir}/ffitarget.h $RPM_BUILD_ROOT%{_includedir}/ffitarget-64.h

cat > $RPM_BUILD_ROOT%{_includedir}/ffi.h <<EOF
#if !defined __x86_64__
# include <ffi-32.h>
#endif
#if defined __x86_64__ && defined __LP64__
# include <ffi-64.h>
#endif
#if defined __x86_64__ && defined __ILP32__
# include <ffi-x32.h>
#endif
EOF
cat > $RPM_BUILD_ROOT%{_includedir}/ffitarget.h <<EOF
#if !defined __x86_64__
# include <ffitarget-32.h>
#endif
#if defined __x86_64__ && defined __LP64__
# include <ffitarget-64.h>
#endif
#if defined __x86_64__ && defined __ILP32__
# include <ffitarget-x32.h>
#endif
EOF

%endif

%if %{have_auto_local}
  %{auto_local} -build_cpu=%{_build_cpu} -target_cpu=%{_target_cpu} \
	-MakeLocCross=%{MakeLocCross} -DelLocCrossDir=%{_bindir}
%endif  # have_auto_local

# ------------------- Clean ------------------------------------------
%Clean

%if %{have_auto_local}
  %{auto_local} -build_cpu=%{_build_cpu} -target_cpu=%{_target_cpu} \
	-MakeLocCross=%{MakeLocCross} \
	-ALD_Ver=%{ALD_Ver} -RpmMovePath=%{DestDir} -tar_arg=bzip2
%endif  # have_auto_local

# ------------------- Files ------------------------------------------
%files
%{_libdir}/*.so.*
%{docdir}

%files devel
%{_includedir}
%{_libdir}/*.so
%{_libdir}/pkgconfig

%files info
%{_infodir}

%files devel-man
%{_mandir}/man3
